MODULE TestFTmini;

	(* Experiments on FreeType for further porting HostFonts *)
	
	IMPORT (*Fonts,*) Ports, Lib := LibsFreeType, Log := StdLog;
	
	PROCEDURE WriteLn (s: ARRAY OF CHAR);
	BEGIN
		Log.String("->"  + s); Log.Ln
	END WriteLn;
	
	PROCEDURE LogError (code: INTEGER);
	BEGIN
		Log.String("Error #"); Log.Int(code); Log.Ln
	END LogError;
	
	PROCEDURE FontMetrics (face: Lib.FT_Face_Ptr);
		CONST grid = 4096;
		VAR
			error: Lib.FT_Error;
			fontSize,
			a, b, scale: INTEGER;
			asc, dsc: INTEGER;
			isScalable: BOOLEAN;
	BEGIN
		isScalable := Lib.FT_FACE_FLAG_SCALABLE IN face.face_flags;
		asc := (face.height + face.descender) * 64;
		dsc := (face.height - face.ascender) * 64;
		
		Log.String("Family name: "); Log.String(face.family_name$); Log.Ln;
		IF isScalable  THEN
			Log.String("SCALABLE"); Log.Ln
		ELSE
			Log.String("NON-SCALABLE"); Log.Ln
		END;
		Log.String("Number of glyphs: "); Log.Int(face.num_glyphs); Log.Ln;
		Log.String("Height: "); Log.Int(face.height * 64); Log.Ln;
		Log.String("BB ascender: "); Log.Int(asc); Log.Ln;
		Log.String("BB descender: "); Log.Int(dsc); Log.Ln;
		Log.String("Grid units: "); Log.Int(face.units_per_EM * 64); Log.Ln;
		
		fontSize := 10 * Ports.point;
		a := fontSize MOD grid; b := fontSize DIV grid;
		scale := a DIV grid + b;
		Log.String("Scale: "); Log.Int(scale); Log.Ln;
	END FontMetrics;
	
	PROCEDURE MeasureWidth (IN text: ARRAY OF CHAR; face: Lib.FT_Face_Ptr);
		VAR
			char: CHAR;
			pos, glyph_index, width: INTEGER;
			error: Lib.FT_Error;
			slot: Lib.FT_GlyphSlot_Ptr;
	BEGIN
		ASSERT(face # NIL);
		pos := 0; char := text[pos];
		WHILE char # 0X DO
			glyph_index := Lib.FT_Get_Char_Index ( face, ORD(char) );
			(*Log.Int(glyph_index); Log.Ln;*)
			ASSERT(face # NIL);
			ASSERT(glyph_index >= 0);
			(*
			error := Lib.FT_Load_Glyph( face, glyph_index, Lib.FT_LOAD_DEFAULT );
			IF error # Lib.FT_Err_Ok THEN LogError(error); HALT(1) END;
			slot := face.glyph; (* load current glyph into slot *)
			width := slot.metrics.horiAdvance; (*	 * 64	*)
			Log.String("Advance: "); Log.Int(width); Log.Ln;
			*)
			INC(pos);
			char := text[pos]
		END
	END MeasureWidth;
	
	PROCEDURE Test*;
		VAR
			(*f: Fonts.Font;*)
			library: Lib.FT_Library_Ptr; (* handle to library *)
			face: Lib.FT_Face_Ptr; (* handle to face object *)
			glyph: Lib.FT_Glyph_Ptr; (* a handle to the glyph image *)
			error: Lib.FT_Error;
			(* filepathname: *)
	BEGIN
		WriteLn("Test");
		
		error := Lib.FT_Init_FreeType( library );
		IF error # Lib.FT_Err_Ok THEN LogError(error); HALT(1) END;
		
		error := Lib.FT_New_Face( library, "tahoma.ttf", 0, face ); (* %windir%\fonts\ *)
		IF error # Lib.FT_Err_Ok THEN LogError(error); HALT(1) END;
		
		FontMetrics( face );

		MeasureWidth( "FreeType", face );
		
		error := Lib.FT_Done_Face( face );
		IF error # Lib.FT_Err_Ok THEN LogError(error); HALT(1) END;
		
		error := Lib.FT_Done_FreeType( library );
		IF error # Lib.FT_Err_Ok THEN LogError(error); HALT(1) END;
		
	END Test;
	
END TestFTmini.

TestFTmini.Test